import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { Inject } from '@nestjs/common';
import { SetConfigurationCommand } from './set-configuration.command';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import {
  ConfigurationDomain,
  ConfigurationIdentifier,
  SetConfigurationRepositoryPort,
} from '@mobicoop/configuration-module';

@CommandHandler(SetConfigurationCommand)
export class SetConfigurationService implements ICommandHandler {
  constructor(
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly configurationRepository: SetConfigurationRepositoryPort,
  ) {}

  async execute(
    command: SetConfigurationCommand,
  ): Promise<ConfigurationIdentifier> {
    return await this.configurationRepository.set(
      {
        domain: command.domain as ConfigurationDomain,
        key: command.key,
      },
      command.value,
    );
  }
}
