import {
  ConfigurationDomain,
  GetConfigurationRepositoryPort,
  NotFoundException,
  SetConfigurationRepositoryPort,
} from '@mobicoop/configuration-module';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { Inject, Injectable, OnApplicationBootstrap } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';

import { CarpoolConfig } from '@src/config/carpool.config';
import { PaginationConfig } from '@src/config/pagination.config';

@Injectable()
export class PopulateService implements OnApplicationBootstrap {
  constructor(
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly getConfigurationRepository: GetConfigurationRepositoryPort,
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly setConfigurationRepository: SetConfigurationRepositoryPort,
    private readonly configService: ConfigService,
  ) {}

  onApplicationBootstrap() {
    this._populate();
  }

  private _populate = async (): Promise<void> => {
    const carpoolConfig: CarpoolConfig = this.configService.get<CarpoolConfig>(
      'carpool',
    ) as CarpoolConfig;
    const paginationConfig: PaginationConfig =
      this.configService.get<PaginationConfig>(
        'pagination',
      ) as PaginationConfig;
    await Promise.all([
      this._populateConfig(ConfigurationDomain.CARPOOL, carpoolConfig),
      this._populateConfig(ConfigurationDomain.PAGINATION, paginationConfig),
    ]);
  };

  private _populateConfig = async <T>(
    domain: ConfigurationDomain,
    config: T,
  ): Promise<void> => {
    let key: keyof typeof config;
    for (key in config) {
      try {
        await this.getConfigurationRepository.get({
          domain,
          key,
        });
      } catch (error: any) {
        if (error instanceof NotFoundException) {
          this.setConfigurationRepository.set(
            {
              domain,
              key,
            },
            `${config[key]}`,
          );
        }
      }
    }
  };
}
