import { Controller, UsePipes } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { RpcExceptionCode, RpcValidationPipe } from '@mobicoop/ddd-library';
import { GRPC_SERVICE_NAME } from '@src/app.constants';
import { SetConfigurationRequestDto } from './dtos/set-configuration.request.dto';
import { SetConfigurationCommand } from '@modules/configuration/core/application/commands/set-configuration/set-configuration.command';
import { ConfigurationIdentifier } from '@mobicoop/configuration-module';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class SetConfigurationGrpcController {
  constructor(private readonly commandBus: CommandBus) {}

  @GrpcMethod(GRPC_SERVICE_NAME, 'Set')
  async set(
    setConfigurationRequestDto: SetConfigurationRequestDto,
  ): Promise<ConfigurationIdentifier> {
    try {
      const configurationIdentifier: ConfigurationIdentifier =
        await this.commandBus.execute(
          new SetConfigurationCommand(setConfigurationRequestDto),
        );
      return configurationIdentifier;
    } catch (error: any) {
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: error.message,
      });
    }
  }
}
