import { NotFoundException } from '@mobicoop/configuration-module';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { PopulateService } from '@modules/configuration/core/application/services/populate.service';
import { ConfigService } from '@nestjs/config';
import { Test, TestingModule } from '@nestjs/testing';

const mockConfigurationRepository = {
  get: jest
    .fn()
    .mockImplementationOnce(() => 'someValue')
    .mockImplementationOnce(() => {
      throw new NotFoundException('Configuration not found');
    }),
  set: jest.fn(),
};

const mockConfigService = {
  get: jest.fn().mockImplementation((domain: string) => {
    switch (domain) {
      case 'carpool':
        return {
          departureTimeMargin: 900,
          role: 'passenger',
          seatsProposed: 3,
          seatsRequested: 1,
          strictFrequency: false,
        };
      case 'pagination':
        return {
          perPage: 10,
        };
    }
  }),
};

describe('Populate Service', () => {
  let populateService: PopulateService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CONFIGURATION_REPOSITORY,
          useValue: mockConfigurationRepository,
        },
        {
          provide: ConfigService,
          useValue: mockConfigService,
        },
        PopulateService,
      ],
    }).compile();

    populateService = module.get<PopulateService>(PopulateService);
  });

  it('should be defined', () => {
    expect(populateService).toBeDefined();
  });

  it('should populate database with default values', () => {
    jest.spyOn(mockConfigurationRepository, 'get');
    jest.spyOn(mockConfigurationRepository, 'set');
    populateService.onApplicationBootstrap();
    expect(mockConfigurationRepository.get).toHaveBeenCalled();
    expect(mockConfigurationRepository.set).toHaveBeenCalled();
  });
});
